﻿using System.Collections.Immutable;
using Connected.Security;
using Connected.Security.Permissions;
using Connected.ServiceModel;
using Connected.Services;
using Connected.Services.Annotations;

namespace Connected.Common.Security.Permissions;

internal class PermissionService : EntityService<int>, IPermissionService
{
	public PermissionService(IContext context) : base(context)
	{
	}

	[ServiceAuthorization(SecurityClaims.SecurityDelete)]
	public async Task Delete(PrimaryKeyArgs<int> args)
	{
		await Invoke(GetOperation<DeletePermission>(), args);
	}

	[ServiceAuthorization(SecurityClaims.SecurityAdd)]
	public async Task<int> Insert(PermissionArgs args)
	{
		return await Invoke(GetOperation<InsertPermission>(), args);
	}

	[ServiceAuthorization(SecurityClaims.SecurityRead, Stage = AuthorizationStage.Result)]
	public async Task<ImmutableList<IPermission>?> Query()
	{
		return await Invoke(GetOperation<QueryPermissions>(), Dto.Empty);
	}

	[ServiceAuthorization(SecurityClaims.SecurityRead, Stage = AuthorizationStage.Result)]
	public async Task<ImmutableList<IPermission>?> Query(PermissionSearchArgs args)
	{
		return await Invoke(GetOperation<SearchPermissions>(), args);
	}

	[ServiceAuthorization(SecurityClaims.SecurityRead, Stage = AuthorizationStage.Result)]
	public async Task<IPermission?> Select(PrimaryKeyArgs<int> args)
	{
		return await Invoke(GetOperation<SelectPermission>(), args);
	}

	[ServiceAuthorization(SecurityClaims.SecurityModify)]
	public async Task Update(PermissionUpdateArgs args)
	{
		await Invoke(GetOperation<UpdatePermission>(), args);
	}
}
